@@ -25,8 +25,8 @@
 #'
 #' # uri with query parameters
 #' (x <- RequestPattern$new(
-#'     method = "get", uri = "https://httpbin.org/get",
-#'     query = list(foo = "bar")
+#'   method = "get", uri = "https://httpbin.org/get",
+#'   query = list(foo = "bar")
 #' ))
 #' x$to_s()
 #' ## query params included in url, not separately
@@ -35,34 +35,64 @@
 #' ))
 #' x$to_s()
 #' x$query_params
-#' 
+#'
 #' # just headers (via setting method=any & uri_regex=.+)
 #' headers <- list(
-#'   'User-Agent' = 'Apple',
-#'   'Accept-Encoding' = 'gzip, deflate', 
-#'   'Accept' = 'application/json, text/xml, application/xml, */*')
+#'   "User-Agent" = "Apple",
+#'   "Accept-Encoding" = "gzip, deflate",
+#'   "Accept" = "application/json, text/xml, application/xml, */*"
+#' )
 #' x <- RequestPattern$new(
-#'    method = "any",
-#'    uri_regex = ".+",
-#'    headers = headers)
+#'   method = "any",
+#'   uri_regex = ".+",
+#'   headers = headers
+#' )
 #' x$to_s()
-#' rs <- RequestSignature$new(method = "any", uri = "http://foo.bar", 
-#'   options = list(headers = headers))
+#' rs <- RequestSignature$new(
+#'   method = "any", uri = "http://foo.bar",
+#'   options = list(headers = headers)
+#' )
 #' rs
 #' x$matches(rs)
-#' 
+#'
 #' # body
-#' x <- RequestPattern$new(method = "post", uri = "httpbin.org/post",
-#'   body = list(y = crul::upload(system.file("CITATION"))))
+#' x <- RequestPattern$new(
+#'   method = "post", uri = "httpbin.org/post",
+#'   body = list(y = crul::upload(system.file("CITATION")))
+#' )
 #' x$to_s()
-#' rs <- RequestSignature$new(method = "post", uri = "http://httpbin.org/post",
+#' rs <- RequestSignature$new(
+#'   method = "post", uri = "http://httpbin.org/post",
 #'   options = list(
-#'      body = list(y = crul::upload(system.file("CITATION")))))
+#'     body = list(y = crul::upload(system.file("CITATION")))
+#'   )
+#' )
 #' rs
 #' x$matches(rs)
+#'
+#' # basic auth
+#' x <- RequestPattern$new(
+#'   method = "post",
+#'   uri = "httpbin.org/post",
+#'   basic_auth = c("user", "pass")
+#' )
+#' x
+#' x$headers_pattern$to_s()
+#' x$to_s()
+#' rs <- RequestSignature$new(
+#'   method = "post", uri = "http://httpbin.org/post",
+#'   options = list(headers = prep_auth("user:pass"))
+#' )
+#' rs
+#' x$matches(rs) # TRUE
+#' rs <- RequestSignature$new(
+#'   method = "post", uri = "http://httpbin.org/post",
+#'   options = list(headers = prep_auth("user:longpassword"))
+#' )
+#' x$matches(rs) # FALSE
 #' }
 RequestPattern <- R6::R6Class(
-  'RequestPattern',
+  "RequestPattern",
   public = list(
     #' @field method_pattern xxx
     method_pattern = NULL,
@@ -81,12 +111,14 @@ RequestPattern <- R6::R6Class(
     #' @param query (list) query parameters, optional
     #' @param body (list) body request, optional
     #' @param headers (list) headers, optional
+    #' @param basic_auth (list) vector of length 2 (username, password),
+    #' optional
     #' @return A new `RequestPattern` object
     initialize = function(method, uri = NULL, uri_regex = NULL,
-                          query = NULL, body = NULL, headers = NULL) {
-
+                          query = NULL, body = NULL, headers = NULL,
+                          basic_auth = NULL) {
       if (is.null(uri) && is.null(uri_regex)) {
-        stop("one of uri or uri_regex is required", call. = FALSE)
+        abort("one of uri or uri_regex is required")
       }
 
       self$method_pattern <- MethodPattern$new(pattern = method)
@@ -97,23 +129,33 @@ RequestPattern <- R6::R6Class(
       }
       self$uri_pattern$add_query_params(query)
       self$body_pattern <- if (!is.null(body)) BodyPattern$new(pattern = body)
-      self$headers_pattern <- if (!is.null(headers))
+      auth_headers <- private$set_basic_auth_as_headers(basic_auth)
+      headers <- c(headers, auth_headers)
+      self$headers_pattern <- if (!is.null(headers)) {
         HeadersPattern$new(pattern = headers)
-      # FIXME: all private methods used in the below line, see if needed or remove
-      # if (length(options)) private$assign_options(options)
+      }
     },
 
     #' @description does a request signature match the selected matchers?
     #' @param request_signature a [RequestSignature] object
     #' @return a boolean
     matches = function(request_signature) {
-      assert(request_signature, "RequestSignature")
-      c_type <- if (!is.null(request_signature$headers)) request_signature$headers$`Content-Type` else NULL
-      if (!is.null(c_type)) c_type <- strsplit(c_type, ';')[[1]][1]
+      assert_is(request_signature, "RequestSignature")
+      c_type <- NULL
+      c_type <- if (!is.null(request_signature$headers)) {
+        request_signature$headers$`Content-Type`
+      }
+      if (!is.null(c_type)) c_type <- strsplit(c_type, ";")[[1]][1]
       self$method_pattern$matches(request_signature$method) &&
         self$uri_pattern$matches(request_signature$uri) &&
-        (is.null(self$body_pattern) || self$body_pattern$matches(request_signature$body, c_type %||% "")) &&
-        (is.null(self$headers_pattern) || self$headers_pattern$matches(request_signature$headers))
+        (
+          is.null(self$body_pattern) ||
+            self$body_pattern$matches(request_signature$body, c_type %||% "")
+        ) &&
+        (
+          is.null(self$headers_pattern) ||
+            self$headers_pattern$matches(request_signature$headers)
+        )
     },
 
     #' @description Print pattern for easy human consumption
@@ -122,56 +164,36 @@ RequestPattern <- R6::R6Class(
       gsub("^\\s+|\\s+$", "", paste(
         toupper(self$method_pattern$to_s()),
         self$uri_pattern$to_s(),
-        if (!is.null(self$body_pattern)) paste0(" with body ", self$body_pattern$to_s()),
-        if (!is.null(self$headers_pattern)) paste0(" with headers ", self$headers_pattern$to_s())
+        if (!is.null(self$body_pattern)) {
+          if (!is.null(self$body_pattern$pattern)) {
+            paste0(" with body ", self$body_pattern$to_s())
+          }
+        },
+        if (!is.null(self$headers_pattern)) {
+          paste0(" with headers ", self$headers_pattern$to_s())
+        }
       ))
     }
   ),
-
   private = list(
-    # assign_options = function(options) {
-    #   #self$validate_keys(options, 'body', 'headers', 'query', 'basic_auth')
-    #   set_basic_auth_as_headers(options)
-    #   self$body_pattern <- if ('body' %in% names(options)) BodyPattern$new(options['body'])
-    #   self$headers_pattern <- if ('headers' %in% names(options)) HeadersPattern$new(options['headers'])
-    #   if ('query' %in% names(options)) self$uri_pattern$add_query_params(options['query'])
-    # },
-
-    # validate_keys = function(x, ...) {
-    #   valid_keys <- unlist(list(...), recursive = FALSE)
-    #   for (i in seq_along(x)) {
-    #     if (!names(x)[i] %in% valid_keys) {
-    #       stop(
-    #         sprintf("Unknown key: %s. Valid keys are: %s",
-    #                 names(x)[i],
-    #                 paste0(valid_keys, collapse = ", "),
-    #                 call. = FALSE
-    #         )
-    #       )
-    #     }
-    #   }
-    # },
-
-    set_basic_auth_as_headers = function(options) {
-      if ('basic_auth' %in% names(options)) {
-        private$validate_basic_auth(options$basic_auth)
-        options$headers <- list()
-        options$headers$Authorization <-
-          private$make_basic_auth(options$basic_auth[1], options$basic_auth[2])
-      }
-    },
-
-    validate_basic_auth = function(x) {
-      if (!inherits(x, "list") || length(unique(unname(unlist(x)))) == 1) {
-        stop(
-          "'basic_auth' option should be a list of length 2: username and password",
-          call. = FALSE
+    set_basic_auth_as_headers = function(x) {
+      if (!is_null(x)) {
+        private$validate_basic_auth(x)
+        list(
+          Authorization = private$make_basic_auth(x[1], x[2])
         )
       }
     },
-
+    validate_basic_auth = function(x) {
+      if (!inherits(x, "character") || length(unique(unname(unlist(x)))) == 1) {
+        abort(c(
+          "error in basic auth",
+          "'basic_auth' option should be a length 2 vector"
+        ))
+      }
+    },
     make_basic_auth = function(x, y) {
-      jsonlite::base64_enc(paste0(x, ":", y))
+      paste0("Basic ", jsonlite::base64_enc(paste0(x, ":", y)))
     }
   )
 )
@@ -186,7 +208,7 @@ RequestPattern <- R6::R6Class(
 #' x$pattern
 #' x$matches(method = "post")
 #' x$matches(method = "POST")
-#' 
+#'
 #' # all matches() calls should be TRUE
 #' (x <- MethodPattern$new(pattern = "any"))
 #' x$pattern
@@ -194,7 +216,7 @@ RequestPattern <- R6::R6Class(
 #' x$matches(method = "GET")
 #' x$matches(method = "HEAD")
 MethodPattern <- R6::R6Class(
-  'MethodPattern',
+  "MethodPattern",
   public = list(
     #' @field pattern (character) an http method
     pattern = NULL,
@@ -245,17 +267,18 @@ MethodPattern <- R6::R6Class(
 #' x$pattern
 #' x$matches(list(`hello-world` = "yep"))
 #' x$matches(list(`hello-worlds` = "yep"))
-#' 
+#'
 #' headers <- list(
-#'   'User-Agent' = 'Apple',
-#'   'Accept-Encoding' = 'gzip, deflate', 
-#'   'Accept' = 'application/json, text/xml, application/xml, */*')
+#'   "User-Agent" = "Apple",
+#'   "Accept-Encoding" = "gzip, deflate",
+#'   "Accept" = "application/json, text/xml, application/xml, */*"
+#' )
 #' (x <- HeadersPattern$new(pattern = headers))
 #' x$to_s()
 #' x$pattern
 #' x$matches(headers)
 HeadersPattern <- R6::R6Class(
-  'HeadersPattern',
+  "HeadersPattern",
   public = list(
     #' @field pattern a list
     pattern = NULL,
@@ -277,7 +300,9 @@ HeadersPattern <- R6::R6Class(
       if (self$empty_headers(self$pattern)) {
         self$empty_headers(headers)
       } else {
-        if (self$empty_headers(headers)) return(FALSE)
+        if (self$empty_headers(headers)) {
+          return(FALSE)
+        }
         headers <- private$normalize_headers(headers)
         out <- c()
         for (i in seq_along(self$pattern)) {
@@ -299,19 +324,23 @@ HeadersPattern <- R6::R6Class(
     #' @return a string
     to_s = function() hdl_lst2(self$pattern)
   ),
-
   private = list(
     normalize_headers = function(x) {
       # normalize names
       names(x) <- tolower(names(x))
-      # normalize symbols
-      ## underscores to single dash
+      # underscores to single dash
       names(x) <- gsub("_", "-", names(x))
       return(x)
     }
   )
 )
 
+#' @importFrom jsonlite fromJSON
+seems_like_json <- function(x) {
+  res <- tryCatch(jsonlite::fromJSON(x), error = function(msg) msg)
+  !inherits(res, "error")
+}
+
 #' @title BodyPattern
 #' @description body matcher
 #' @export
@@ -335,53 +364,105 @@ HeadersPattern <- R6::R6Class(
 #' z <- BodyPattern$new(pattern = list(foo = "bar", a = 5))
 #' z$pattern
 #' z$matches(bb$body)
-#' 
+#'
 #' # uploads in bodies
 #' ## upload NOT in a list
 #' bb <- RequestSignature$new(
 #'   method = "post", uri = "https:/httpbin.org/post",
-#'   options = list(body = crul::upload(system.file("CITATION"))))
+#'   options = list(body = crul::upload(system.file("CITATION")))
+#' )
 #' bb$body
-#' z <- BodyPattern$new(pattern = 
-#'   crul::upload(system.file("CITATION")))
+#' z <- BodyPattern$new(
+#'   pattern =
+#'     crul::upload(system.file("CITATION"))
+#' )
 #' z$pattern
 #' z$matches(bb$body)
-#' 
+#'
 #' ## upload in a list
 #' bb <- RequestSignature$new(
 #'   method = "post", uri = "https:/httpbin.org/post",
-#'   options = list(body = list(y = crul::upload(system.file("CITATION")))))
+#'   options = list(body = list(y = crul::upload(system.file("CITATION"))))
+#' )
 #' bb$body
-#' z <- BodyPattern$new(pattern =
-#'   list(y = crul::upload(system.file("CITATION"))))
+#' z <- BodyPattern$new(
+#'   pattern =
+#'     list(y = crul::upload(system.file("CITATION")))
+#' )
 #' z$pattern
 #' z$matches(bb$body)
+#'
+#' # partial matching
+#' ## including
+#' partial_incl <- including(list(foo = "bar"))
+#' z <- BodyPattern$new(pattern = partial_incl)
+#' z$pattern
+#' z$matches(list(foo = "bar", a = 5)) # TRUE
+#'
+#' ## excluding
+#' partial_excl <- excluding(list(hello = "world"))
+#' z <- BodyPattern$new(pattern = partial_excl)
+#' z$pattern
+#' z$matches(list(a = 5)) # TRUE
+#' z$matches(list(hello = "mars", a = 5)) # TRUE
+#' z$matches(list(hello = "world")) # FALSE
 BodyPattern <- R6::R6Class(
-  'BodyPattern',
+  "BodyPattern",
   public = list(
     #' @field pattern a list
     pattern = NULL,
+    #' @field partial bool, default: `FALSE`
+    partial = FALSE,
+    #' @field partial_type a string, default: NULL
+    partial_type = NULL,
 
     #' @description Create a new `BodyPattern` object
-    #' @param pattern (list) a body object
+    #' @param pattern (list) a body object - from a request stub (i.e.,
+    #' the mock)
     #' @return A new `BodyPattern` object
     initialize = function(pattern) {
-      if (inherits(pattern, "form_file")) 
+      if (inherits(pattern, "partial")) {
+        self$partial <- attr(pattern, "partial_match") %||% FALSE
+        self$partial_type <- attr(pattern, "partial_type")
+        pattern <- drop_partial_attrs(pattern)
         self$pattern <- unclass(pattern)
-      else 
+      } else if (inherits(pattern, "form_file")) {
+        self$pattern <- unclass(pattern)
+      } else {
         self$pattern <- pattern
+      }
+
+      # convert self$pattern to a list if it's json
+      if (seems_like_json(self$pattern)) {
+        self$pattern <- jsonlite::fromJSON(self$pattern, FALSE)
+      }
     },
 
+    #' @importFrom rlang is_null is_na
     #' @description Match a request body pattern against a pattern
-    #' @param body (list) the body
+    #' @param body (list) the body, i.e., from the HTTP request
     #' @param content_type (character) content type
     #' @return a boolean
     matches = function(body, content_type = "") {
       if (inherits(self$pattern, "list")) {
-        if (length(self$pattern) == 0) return(TRUE)
-        private$matching_hashes(private$body_as_hash(body, content_type), self$pattern)
+        if (length(self$pattern) == 0) {
+          return(TRUE)
+        }
+        private$matching_hashes(
+          self$pattern,
+          private$body_as_hash(body, content_type)
+        )
       } else {
-        (private$empty_string(self$pattern) && private$empty_string(body)) || all(self$pattern == body)
+        # FIXME: add partial approach later
+        (private$empty_string(self$pattern) && private$empty_string(body)) || {
+          if (xor(is_na(self$pattern), is_na(body))) {
+            return(FALSE)
+          }
+          if (xor(is_null(self$pattern), is_null(body))) {
+            return(FALSE)
+          }
+          all(self$pattern == body)
+        }
       }
     },
 
@@ -389,37 +470,80 @@ BodyPattern <- R6::R6Class(
     #' @return a string
     to_s = function() self$pattern
   ),
-
   private = list(
     empty_string = function(string) {
-      is.null(string) || !nzchar(string)
+      is_null(string) || !nzchar(string)
     },
+    matching_hashes = function(pattern, body) {
+      if (is_null(pattern)) {
+        return(FALSE)
+      }
+      if (!inherits(pattern, "list")) {
+        return(FALSE)
+      }
+      if (!rlang::is_list(body)) {
+        return(FALSE)
+      }
 
-    matching_hashes = function(z, pattern) {
-      if (is.null(z)) return(FALSE)
-      if (!inherits(z, "list")) return(FALSE)
-      if (!all(sort(names(z)) %in% sort(names(pattern)))) return(FALSE)
-      for (i in seq_along(z)) {
-        expected <- pattern[[names(z)[i]]]
-        actual <- z[[i]]
-        if (inherits(actual, "list") && inherits(expected, "list")) {
-          if (!private$matching_hashes(actual, expected)) return(FALSE)
-        } else {
-          if (!identical(as.character(actual), as.character(expected))) return(FALSE)
+      pattern_char <- rapply(pattern, as.character, how = "replace")
+      body_char <- rapply(body, as.character, how = "replace")
+
+      if (self$partial) {
+        names_values_check <- switch(self$partial_type,
+          # unname() here not needed for R < 4.5, but is needed for R 4.5
+          # because intersect changes to output unnamed lists
+          include =
+            identical(
+              unname(intersect(pattern_char, body_char)),
+              unname(pattern_char)
+            ),
+          exclude = length(intersect(pattern_char, body_char)) == 0
+        )
+        if (!names_values_check) {
+          return(FALSE)
+        }
+      } else {
+        if (!identical(pattern_char, body_char)) {
+          return(FALSE)
         }
       }
+
+      # return TRUE (a match) if no FALSE's returned above
       return(TRUE)
     },
-
     body_as_hash = function(body, content_type) {
       if (inherits(body, "form_file")) body <- unclass(body)
+      if (is_empty(content_type)) content_type <- ""
       bctype <- BODY_FORMATS[[content_type]] %||% ""
-      if (bctype == 'json') {
+      if (grepl("json", content_type)) {
+        bctype <- "json"
+      }
+      if (bctype == "json") {
         jsonlite::fromJSON(body, FALSE)
-      } else if (bctype == 'xml') {
-        check_for_pkg("xml2")
-        xml2::read_xml(body)
+      } else if (bctype == "xml") {
+        check_installed("xml2")
+        try_xml2list <- rlang::try_fetch(
+          {
+            body_xml <- xml2::read_xml(body)
+            xml_as_list <- xml2::as_list(body_xml)
+            lapply(xml_as_list, promote_attr)
+          },
+          error = function(e) e
+        )
+        if (rlang::is_error(try_xml2list)) {
+          rlang::warn(
+            "xml to list conversion failed; using xml string for comparison",
+            use_cli_format = TRUE,
+            .frequency = "always"
+          )
+          body
+        } else {
+          try_xml2list
+        }
       } else {
+        if (seems_like_json(body)) {
+          return(jsonlite::fromJSON(body, FALSE))
+        }
         query_mapper(body)
       }
     }
@@ -427,18 +551,46 @@ BodyPattern <- R6::R6Class(
 )
 
 BODY_FORMATS <- list(
-  'text/xml'               = 'xml',
-  'application/xml'        = 'xml',
-  'application/json'       = 'json',
-  'text/json'              = 'json',
-  'application/javascript' = 'json',
-  'text/javascript'        = 'json',
-  'text/html'              = 'html',
-  'application/x-yaml'     = 'yaml',
-  'text/yaml'              = 'yaml',
-  'text/plain'             = 'plain'
+  "text/xml"                   = "xml",
+  "application/xml"            = "xml",
+  "application/json"           = "json",
+  "text/json"                  = "json",
+  "application/javascript"     = "json",
+  "text/javascript"            = "json",
+  "application/x-amz-json-1.1" = "json", # AWS
+  "text/html"                  = "html",
+  "application/x-yaml"         = "yaml",
+  "text/yaml"                  = "yaml",
+  "text/plain"                 = "plain"
 )
 
+# remove_reserved & promote_attr from
+# https://www.garrickadenbuie.com/blog/recursive-xml-workout/
+remove_reserved <- function(this_attr) {
+  reserved_attr <- c(
+    "class", "comment", "dim", "dimnames",
+    "names", "row.names", "tsp"
+  )
+  if (!any(reserved_attr %in% names(this_attr))) {
+    return(this_attr)
+  }
+  for (reserved in reserved_attr) {
+    if (!is.null(this_attr[[reserved]])) this_attr[[reserved]] <- NULL
+  }
+  this_attr
+}
+promote_attr <- function(ll) {
+  this_attr <- attributes(ll)
+  this_attr <- remove_reserved(this_attr)
+  if (length(ll)) {
+    # recursive case
+    c(this_attr, lapply(ll, promote_attr))
+  } else {
+    # base case (no sub-items)
+    this_attr
+  }
+}
+
 #' @title UriPattern
 #' @description uri matcher
 #' @export
@@ -448,7 +600,7 @@ BODY_FORMATS <- list(
 #' (z <- UriPattern$new(pattern = "http://foobar.com"))
 #' z$matches("http://foobar.com") # TRUE
 #' z$matches("http://foobar.com/") # TRUE
-#' 
+#'
 #' # without scheme
 #' ## matches http by default: does not match https by default
 #' (z <- UriPattern$new(pattern = "foobar.com"))
@@ -485,13 +637,13 @@ BODY_FORMATS <- list(
 #' z$pattern
 #' z$matches("http://foobar.com?pizza=cheese&cheese=cheddar") # TRUE
 #' z$matches("http://foobar.com?pizza=cheese&cheese=swiss") # FALSE
-#' 
+#'
 #' # query parameters in the uri
 #' (z <- UriPattern$new(pattern = "https://httpbin.org/get?stuff=things"))
 #' z$add_query_params() # have to run this method to gather query params
 #' z$matches("https://httpbin.org/get?stuff=things") # TRUE
 #' z$matches("https://httpbin.org/get?stuff2=things") # FALSE
-#' 
+#'
 #' # regex add query parameters
 #' (z <- UriPattern$new(regex_pattern = "https://foobar.com/.+/order"))
 #' z$add_query_params(list(pizza = "cheese"))
@@ -499,13 +651,13 @@ BODY_FORMATS <- list(
 #' z$pattern
 #' z$matches("https://foobar.com/pizzas/order?pizza=cheese") # TRUE
 #' z$matches("https://foobar.com/pizzas?pizza=cheese") # FALSE
-#' 
+#'
 #' # query parameters in the regex uri
 #' (z <- UriPattern$new(regex_pattern = "https://x.com/.+/order\\?fruit=apple"))
 #' z$add_query_params() # have to run this method to gather query params
 #' z$matches("https://x.com/a/order?fruit=apple") # TRUE
 #' z$matches("https://x.com/a?fruit=apple") # FALSE
-#' 
+#'
 #' # any pattern
 #' (z <- UriPattern$new(regex_pattern = "stuff\\.com.+"))
 #' z$regex
@@ -513,9 +665,28 @@ BODY_FORMATS <- list(
 #' z$matches("http://stuff.com") # FALSE
 #' z$matches("https://stuff.com/stff") # TRUE
 #' z$matches("https://stuff.com/apple?bears=brown&bats=grey") # TRUE
-
+#'
+#' # partial matching
+#' ## including
+#' z <- UriPattern$new(pattern = "http://foobar.com")
+#' z$add_query_params(including(list(hello = "world")))
+#' z$matches(uri = "http://foobar.com?hello=world&bye=mars") # TRUE
+#' z$matches("http://foobar.com?bye=mars") # FALSE
+#'
+#' ## excluding
+#' z <- UriPattern$new(pattern = "http://foobar.com")
+#' z$add_query_params(excluding(list(hello = "world")))
+#' z$matches(uri = "http://foobar.com?hello=world&bye=mars") # FALSE
+#' z$matches("http://foobar.com?bye=mars") # TRUE
+#'
+#' ## match on list keys (aka: names) only, ignore values 0
+#' z <- UriPattern$new(pattern = "http://foobar.com")
+#' z$add_query_params(including(list(hello = NULL)))
+#' z$matches(uri = "http://foobar.com?hello=world&bye=mars") # TRUE
+#' z$matches("http://foobar.com?hello=stuff") # TRUE
+#' z$matches("http://foobar.com?bye=stuff") # FALSE
 UriPattern <- R6::R6Class(
-  'UriPattern',
+  "UriPattern",
   public = list(
     #' @field pattern (character) pattern holder
     pattern = NULL,
@@ -523,6 +694,10 @@ UriPattern <- R6::R6Class(
     regex = FALSE,
     #' @field query_params a list, or `NULL` if empty
     query_params = NULL,
+    #' @field partial bool, default: `FALSE`
+    partial = FALSE,
+    #' @field partial_type a string, default: NULL
+    partial_type = NULL,
 
     #' @description Create a new `UriPattern` object
     #' @param pattern (character) a uri, as a character string. if scheme
@@ -544,24 +719,46 @@ UriPattern <- R6::R6Class(
     #' @return a boolean
     matches = function(uri) {
       uri <- normalize_uri(uri, self$regex)
-      if (self$regex)
+      if (self$regex) {
         grepl(self$pattern, uri)
-      else
+      } else {
         self$pattern_matches(uri) && self$query_params_matches(uri)
+      }
     },
 
     #' @description Match a URI
     #' @param uri (character) a uri
     #' @return a boolean
     pattern_matches = function(uri) {
-      if (!self$regex) return(uri == self$pattern) # not regex
-      grepl(drop_query_params(self$pattern), uri) # regex
+      if (!self$regex) {
+        return(just_uri(uri) == just_uri(self$pattern))
+      } # not regex
+      grepl(drop_query_params(self$pattern), just_uri(uri)) # regex
     },
 
+    #' @importFrom rlang is_empty
     #' @description Match query parameters of a URI
     #' @param uri (character) a uri
     #' @return a boolean
     query_params_matches = function(uri) {
+      if (self$partial) {
+        uri_qp <- self$extract_query(uri)
+        qp <- drop_partial_attrs(self$query_params)
+
+        bools <- vector(mode = "logical")
+        for (i in seq_along(qp)) {
+          if (rlang::is_empty(qp[[i]])) {
+            bools[i] <- names(qp) %in% names(uri_qp)
+          } else {
+            bools[i] <- qp %in% uri_qp
+          }
+        }
+        out <- switch(self$partial_type,
+          include = any(bools),
+          exclude = !any(bools)
+        )
+        return(out)
+      }
       identical(self$query_params, self$extract_query(uri))
     },
 
@@ -570,7 +767,9 @@ UriPattern <- R6::R6Class(
     #' @return named list, or `NULL` if no query parameters
     extract_query = function(uri) {
       params <- parse_a_url(uri)$parameter
-      if (all(is.na(params))) return(NULL)
+      if (all(is.na(params))) {
+        return(NULL)
+      }
       params
     },
 
@@ -578,17 +777,23 @@ UriPattern <- R6::R6Class(
     #' @param query_params (list|character) list or character
     #' @return nothing returned, updates uri pattern
     add_query_params = function(query_params) {
-      if (self$regex) return(NULL)
+      if (self$regex) {
+        return(NULL)
+      }
       if (missing(query_params) || is.null(query_params)) {
         self$query_params <- self$extract_query(self$pattern)
       } else {
         self$query_params <- query_params
+        self$partial <- attr(query_params, "partial_match") %||% FALSE
+        self$partial_type <- attr(query_params, "partial_type")
         if (
           inherits(query_params, "list") ||
-          inherits(query_params, "character")
+            inherits(query_params, "character")
         ) {
-          pars <- paste0(unname(Map(function(x, y) paste(x, esc(y), sep = "="),
-              names(query_params), query_params)), collapse = "&")
+          pars <- paste0(unname(Map(
+            function(x, y) paste(x, esc(y), sep = "="),
+            names(query_params), query_params
+          )), collapse = "&")
           self$pattern <- paste0(self$pattern, "?", pars)
         }
       }
@@ -600,9 +805,15 @@ UriPattern <- R6::R6Class(
   )
 )
 
+drop_partial_attrs <- function(x) {
+  attr(x, "partial_match") <- NULL
+  attr(x, "partial_type") <- NULL
+  return(x)
+}
+
 add_scheme <- function(x) {
   if (is.na(urltools::url_parse(x)$scheme)) {
-    paste0('https?://', x)
+    paste0("https?://", x)
   } else {
     x
   }
@@ -611,11 +822,15 @@ esc <- function(x) curl::curl_escape(x)
 normalize_uri <- function(x, regex = FALSE) {
   x <- prune_trailing_slash(x)
   x <- prune_port(x)
-  if (!regex) 
-    if (is.na(urltools::url_parse(x)$scheme))
-      x <- paste0('http://', x)
+  if (!regex) {
+    if (is.na(urltools::url_parse(x)$scheme)) {
+      x <- paste0("http://", x)
+    }
+  }
   tmp <- urltools::url_parse(x)
-  if (is.na(tmp$path)) return(x)
+  if (is.na(tmp$path)) {
+    return(x)
+  }
   if (!regex) tmp$path <- esc(tmp$path)
   urltools::url_compose(tmp)
 }
@@ -644,7 +859,8 @@ parse_a_url <- function(url) {
         strsplit(tmp$parameter, "&")[[1]], function(x) {
           z <- strsplit(x, split = "=")[[1]]
           as.list(stats::setNames(z[2], z[1]))
-        }),
+        }
+      ),
       recursive = FALSE
     )
   }
@@ -652,6 +868,12 @@ parse_a_url <- function(url) {
   return(tmp)
 }
 
+just_uri <- function(x) {
+  z <- urltools::url_parse(x)
+  z$parameter <- NA_character_
+  urltools::url_compose(z)
+}
+
 uri_fetch <- function(x) {
   x <- as.character(x)
   tmp <- x[vapply(x, FUN = is_url, FUN.VALUE = logical(1))]
@@ -668,30 +890,3 @@ drop_query_params <- function(x) {
   # prune trailing slash
   sub("\\/$", "", x)
 }
-
-## http method
-get_method <- function(x) {
-  x <- as.character(x)
-  tmp <- grep(
-    "(get)$|(post)$|(put)$|(delete)$|(options)$|(patch)$|(head)$",
-    tolower(x), value = TRUE)
-  tmp <- sub("httr::", "", tmp)
-  if (length(tmp) == 0) NULL else tmp
-}
-
-## query and body stuff
-get_query <- function(x) {
-  if ("query" %in% names(x)) {
-    x[["query"]]
-  } else {
-    NULL
-  }
-}
-
-get_body <- function(x) {
-  if ("body" %in% names(x)) {
-    x[["body"]]
-  } else {
-    NULL
-  }
-}
