@@ -3,7 +3,7 @@ context("RequestSignature")
 test_that("RequestSignature: works", {
   expect_is(RequestSignature, "R6ClassGenerator")
 
-  aa <- RequestSignature$new(method = "get", uri = "https:/httpbin.org/get")
+  aa <- RequestSignature$new(method = "get", uri = hb("/get"))
 
   expect_is(aa, "RequestSignature")
 
@@ -18,15 +18,31 @@ test_that("RequestSignature: works", {
   expect_equal(aa$method, "get")
 
   expect_is(aa$uri, "character")
-  expect_equal(aa$uri, "https:/httpbin.org/get")
+  expect_equal(aa$uri, hb("/get"))
 
   expect_is(aa$to_s, "function")
-  expect_equal(aa$to_s(), "GET:  https:/httpbin.org/get")
+  expect_equal(aa$to_s(), sprintf("GET:  %s", hb("/get")))
+})
+
+test_that("RequestSignature: with bodies work", {
+  aa <- RequestSignature$new(
+    method = "get", uri = hb("/get"),
+    options = list(body = "abc")
+  )
+  expect_output(print(aa), "<unnamed>")
+
+  bb <- RequestSignature$new(
+    method = "get", uri = hb("/get"),
+    options = list(body = list(some_thing = "abc"))
+  )
+  expect_no_match(capture.output(print(bb)), "<unnamed>")
 })
 
 test_that("RequestSignature: different methods work", {
-  aa <- RequestSignature$new(method = "post", uri = "https:/httpbin.org/post",
-    options = list(fields = list(foo = "bar")))
+  aa <- RequestSignature$new(
+    method = "post", uri = hb("/post"),
+    options = list(fields = list(foo = "bar"))
+  )
   aa$headers <- list(Accept = "application/json")
   aa$body <- list(foo = "bar")
 
@@ -40,8 +56,12 @@ test_that("RequestSignature: different methods work", {
 
 test_that("RequestSignature fails well", {
   expect_error(RequestSignature$new(), "argument \"method\" is missing")
-  expect_error(RequestSignature$new(method = "adf"),
-               "'arg' should be one of")
-  expect_error(RequestSignature$new(method = "get"),
-               "argument \"uri\" is missing")
+  expect_error(
+    RequestSignature$new(method = "adf"),
+    "'arg' should be one of"
+  )
+  expect_error(
+    RequestSignature$new(method = "get"),
+    "argument \"uri\" is missing"
+  )
 })
