@@ -7,10 +7,10 @@ test_that("request_registry: structure", {
   expect_is(request_registry(), "RequestRegistry")
 
   enable()
-  stub_request("get", "https://httpbin.org/get") %>%
+  stub_request("get", hb("/get")) %>%
     to_return(body = "success!", status = 200)
   invisible(
-    crul::HttpClient$new(url = "https://httpbin.org")$get("get")
+    crul::HttpClient$new(url = hb())$get("get")
   )
   disable()
 
@@ -25,5 +25,5 @@ test_that("request_registry: structure", {
 
   expect_is(x$request_signatures$hash, "list")
   expect_match(names(x$request_signatures$hash), "GET")
-  expect_is(x$request_signatures$hash[[1]]$count, 'numeric')
+  expect_is(x$request_signatures$hash[[1]]$count, "numeric")
 })
