@@ -51,10 +51,12 @@ test_that("to_return: setting body behaves correctly", {
 })
 
 test_that("to_return: setting body with wrong type errors well", {
+  stub_registry_clear()
+
   ## ERRORS when not of right type
   expect_error(
     stub_request("get", "https://google.com") %>%
       to_return(body = TRUE),
-    "Unknown type of `body`"
+    "Unknown `body` type"
   )
 })
