@@ -6,10 +6,9 @@ test_that("no stubs exist before stub_request called", {
   expect_equal(length(stub_registry()$request_stubs), 0)
 })
 
-aa <- stub_request("get", "https://httpbin.org/get") %>% to_timeout()
+aa <- stub_request("get", hb("/get")) %>% to_timeout()
 
 test_that("stub_request bits are correct", {
-
   expect_is(aa, "StubbedRequest")
   expect_null(aa$body)
   expect_null(aa$host)
@@ -22,7 +21,7 @@ test_that("stub_request bits are correct", {
   expect_is(aa$method, "character")
   expect_equal(aa$method, "get")
   expect_is(aa$uri, "character")
-  expect_equal(aa$uri, "https://httpbin.org/get")
+  expect_equal(aa$uri, hb("/get"))
 
   # to_timeout expected stuff
   expect_true(aa$responses_sequences[[1]]$timeout)
@@ -34,7 +33,7 @@ test_that("stubs exist after stub_request called", {
 
 test_that("stub_request fails well", {
   expect_error(to_timeout(), "argument \".data\" is missing")
-  expect_error(to_timeout(5), ".data must be of class StubbedRequest")
+  expect_error(to_timeout(5), "must be of class StubbedRequest")
 })
 
 # cleanup
