@@ -1,7 +1,7 @@
 context("wi_th")
 
 test_that("wi_th: with just headers", {
-  aa <- stub_request("get", "https://httpbin.org/get") %>%
+  aa <- stub_request("get", hb("/get")) %>%
     wi_th(headers = list("User-Agent" = "R"))
 
   expect_is(aa, "StubbedRequest")
@@ -16,15 +16,16 @@ test_that("wi_th: with just headers", {
   expect_is(aa$method, "character")
   expect_equal(aa$method, "get")
   expect_is(aa$uri, "character")
-  expect_equal(aa$uri, "https://httpbin.org/get")
+  expect_equal(aa$uri, hb("/get"))
   expect_equal(aa$request_headers, list("User-Agent" = "R"))
 })
 
 test_that("wi_th: with headers and query", {
-  aa <- stub_request("get", "https://httpbin.org/get") %>%
+  aa <- stub_request("get", hb("/get")) %>%
     wi_th(
       query = list(hello = "world"),
-      headers = list("User-Agent" = "R"))
+      headers = list("User-Agent" = "R")
+    )
 
   expect_is(aa$query, "list")
   expect_is(aa$request_headers, "list")
@@ -34,36 +35,40 @@ test_that("wi_th: with headers and query", {
 })
 
 test_that("wi_th: bodies", {
-  aa <- stub_request("post", "https://httpbin.org/post") %>%
+  aa <- stub_request("post", hb("/post")) %>%
     wi_th(body = list(foo = "bar"))
   expect_is(aa$body, "list")
   expect_output(print(aa), "body \\(class: list\\): foo=bar")
 
-  bb <- stub_request("post", "https://httpbin.org/post") %>%
+  bb <- stub_request("post", hb("/post")) %>%
     wi_th(body = '{"foo": "bar"}')
   expect_is(bb$body, "character")
-  expect_output(print(bb),
-    "body \\(class: character\\): \\{\"foo\": \"bar\"\\}")
+  expect_output(
+    print(bb),
+    "body \\(class: character\\): \\{\"foo\": \"bar\"\\}"
+  )
 
-  cc <- stub_request("post", "https://httpbin.org/post") %>%
+  cc <- stub_request("post", hb("/post")) %>%
     wi_th(body = charToRaw('{"foo": "bar"}'))
   expect_is(cc$body, "raw")
-  expect_output(print(cc),
-    "body \\(class: raw\\): raw bytes, length: 14")
+  expect_output(
+    print(cc),
+    "body \\(class: raw\\): raw bytes, length: 14"
+  )
 
-  dd <- stub_request("post", "https://httpbin.org/post") %>%
+  dd <- stub_request("post", hb("/post")) %>%
     wi_th(body = 5)
   expect_is(dd$body, "numeric")
   expect_output(print(dd), "body \\(class: numeric\\): 5")
 
-  ee <- stub_request("post", "https://httpbin.org/post") %>%
+  ee <- stub_request("post", hb("/post")) %>%
     wi_th(body = crul::upload(system.file("CITATION")))
   expect_is(ee$body, "form_file")
   expect_output(print(ee), "body \\(class: form_file\\): crul::upload")
 
-  # FIXME: ideally (maybe?) we have a upload within a list look like 
+  # FIXME: ideally (maybe?) we have a upload within a list look like
   # the above when not in a list?
-  ff <- stub_request("post", "https://httpbin.org/post") %>%
+  ff <- stub_request("post", hb("/post")) %>%
     wi_th(body = list(y = crul::upload(system.file("CITATION"))))
   expect_is(ff$body, "list")
   expect_is(ff$body$y, "form_file")
@@ -72,29 +77,42 @@ test_that("wi_th: bodies", {
 
 test_that("wi_th fails well", {
   expect_error(wi_th(), "argument \".data\" is missing")
-  expect_error(wi_th(5), ".data must be of class StubbedRequest")
-
-  zzz <- stub_request("get", "https://httpbin.org/get")
+  expect_error(wi_th(5), "must be of class StubbedRequest")
 
   # query
-  expect_error(wi_th(zzz, query = list(5, 6)),
-               "'query' must be a named list")
-  expect_error(wi_th(zzz, query = list(a = 5, 6)),
-               "'query' must be a named list")
+  zzz <- stub_request("get", hb("/get"))
+  expect_error(
+    sw(wi_th(zzz, query = list(5, 6))),
+    "'query' must be a named list"
+  )
+  zzz <- stub_request("get", hb("/get"))
+  expect_error(
+    sw(wi_th(zzz, query = list(a = 5, 6))),
+    "'query' must be a named list"
+  )
 
   # headers
-  expect_error(wi_th(zzz, headers = list(5, 6)),
-               "'headers' must be a named list")
-  expect_error(wi_th(zzz, headers = list(a = 5, 6)),
-               "'headers' must be a named list")
+  zzz <- stub_request("get", hb("/get"))
+  expect_error(
+    sw(wi_th(zzz, headers = list(5, 6))),
+    "'headers' must be a named list"
+  )
+  zzz <- stub_request("get", hb("/get"))
+  expect_error(
+    sw(wi_th(zzz, headers = list(a = 5, 6))),
+    "'headers' must be a named list"
+  )
 
   # only accepts certain set of named things
-  expect_error(wi_th(zzz, a = 5),
-    "'wi_th' only accepts query, body, headers")
+  zzz <- stub_request("get", hb("/get"))
+  expect_error(
+    sw(wi_th(zzz, a = 5)),
+    "'wi_th' only accepts query, body, headers"
+  )
 })
 
 test_that("wi_th .list works", {
-  req <- stub_request("post", "https://httpbin.org/post")
+  req <- stub_request("post", hb("/post"))
   expect_equal(
     wi_th(req, .list = list(body = list(foo = "bar"))),
     wi_th(req, body = list(foo = "bar"))
@@ -105,9 +123,15 @@ test_that("wi_th .list works", {
   )
   expect_equal(wi_th(req, .list = ), wi_th(req))
 
-  expect_error(wi_th(req, .list = 4), ".list must be of class list")
-  expect_error(wi_th(req, .list = list(a = 5)),
-    "'wi_th' only accepts query, body, headers")
+  expect_error(
+    sw(wi_th(req, .list = 4)),
+    "must be of class list"
+  )
+  req <- stub_request("post", hb("/post"))
+  expect_error(
+    sw(wi_th(req, .list = list(a = 5))),
+    "'wi_th' only accepts query, body, headers"
+  )
 })
 
 # addresses issue: https://github.com/ropensci/webmockr/issues/107
@@ -160,42 +184,42 @@ test_that("wi_th handles HEADERS with varied input classes", {
   # works w/ numeric
   stub_request("get", "https://x.com") %>%
     wi_th(headers = list(foo = 30))
-  expect_is(GET("https://x.com", add_headers(foo=30)), "response")
+  expect_is(GET("https://x.com", add_headers(foo = 30)), "response")
 
   # works w/ integer
   stub_registry_clear()
   stub_request("get", "https://x.com") %>%
     wi_th(headers = list(foo = 30L))
-  expect_is(GET("https://x.com", add_headers(foo=30)), "response")
+  expect_is(GET("https://x.com", add_headers(foo = 30)), "response")
 
   # works w/ character
   stub_registry_clear()
   stub_request("get", "https://x.com") %>%
     wi_th(headers = list(foo = "30"))
-  expect_is(GET("https://x.com", add_headers(foo=30)), "response")
+  expect_is(GET("https://x.com", add_headers(foo = 30)), "response")
 
   # works w/ number as factor
   stub_registry_clear()
   stub_request("get", "https://x.com") %>%
     wi_th(headers = list(foo = as.factor(30)))
-  expect_is(GET("https://x.com", add_headers(foo=30)), "response")
+  expect_is(GET("https://x.com", add_headers(foo = 30)), "response")
 
   # works w/ character as factor
   stub_registry_clear()
   stub_request("get", "https://x.com") %>%
     wi_th(headers = list(foo = as.factor("bar")))
-  expect_is(GET("https://x.com", add_headers(foo="bar")), "response")
+  expect_is(GET("https://x.com", add_headers(foo = "bar")), "response")
 
   # works w/ AsIs
   stub_registry_clear()
   stub_request("get", "https://x.com") %>%
     wi_th(headers = list(foo = 30))
-  expect_is(GET("https://x.com", add_headers(foo=30)), "response")
+  expect_is(GET("https://x.com", add_headers(foo = 30)), "response")
 })
 
 disable("httr")
 
-test_that("wi_th basic_auth", {
+test_that("wi_th basic_auth, crul", {
   # crul
   library(crul)
   enable("crul")
@@ -203,7 +227,7 @@ test_that("wi_th basic_auth", {
   # pass
   stub_registry_clear()
   stub_request("get", "https://x.com") %>%
-    wi_th(basic_auth=c("user", "passwd"))
+    wi_th(basic_auth = c("user", "passwd"))
   expect_is(con$get(), "HttpResponse")
   # ignores auth type
   con$auth <- crul::auth("user", "passwd", "digest")
@@ -211,35 +235,62 @@ test_that("wi_th basic_auth", {
   # fail
   stub_registry_clear()
   stub_request("get", "https://x.com") %>%
-    wi_th(basic_auth=c("user", "passwd"))
+    wi_th(basic_auth = c("user", "passwd"))
   con$auth <- crul::auth("user", "password")
   expect_error(con$get(), "Unregistered")
   disable("crul")
+})
 
-  # httr
+test_that("wi_th basic_auth, httr", {
   library(httr)
   enable("httr")
   # pass
   stub_registry_clear()
   stub_request("get", "https://x.com") %>%
-    wi_th(basic_auth=c("user", "passwd"))
+    wi_th(basic_auth = c("user", "passwd"))
   expect_is(GET("https://x.com", authenticate("user", "passwd")), "response")
   # ignores auth type
   expect_is(
     GET("https://x.com", authenticate("user", "passwd", type = "digest")),
-    "response")
+    "response"
+  )
   expect_is(
     GET("https://x.com", authenticate("user", "passwd", type = "ntlm")),
-    "response")
+    "response"
+  )
   # fail
   stub_registry_clear()
   stub_request("get", "https://x.com") %>%
-    wi_th(basic_auth=c("user", "passwd"))
-  expect_error(GET("https://x.com", authenticate("user", "password")),
-    "Unregistered")
+    wi_th(basic_auth = c("user", "passwd"))
+  expect_error(
+    GET("https://x.com", authenticate("user", "password")),
+    "Unregistered"
+  )
   disable("httr")
 })
 
+test_that("wi_th basic_auth, httr2", {
+  skip_if_not_installed("httr2")
+  library(httr2)
+  enable("httr2")
+  # pass
+  stub_registry_clear()
+  stub_request("get", "https://x.com") %>%
+    wi_th(basic_auth = c("user", "passwd"))
+  req <- request("https://x.com") %>% req_auth_basic("user", "passwd")
+  expect_is(req_perform(req), "httr2_response")
+  # fail
+  stub_registry_clear()
+  stub_request("get", "https://x.com") %>%
+    wi_th(basic_auth = c("user", "passwd"))
+  req2 <- request("https://x.com") %>% req_auth_basic("user", "password")
+  expect_error(
+    req_perform(req2),
+    "Unregistered"
+  )
+  disable("httr2")
+})
+
 # cleanup
 stub_registry_clear()
 
